==> LoadL-3.4.0.1-0.SLES10-X86_64.3.4.X.tar.gz: opt/ibmll/LoadL/full/samples/llcluster/cluster_metric_free_nodes.c <==
/*
 * ----------------------------------------------------------------------------
 * This sample program shows how LoadLeveler's Data Access API can be used
 * to create a cluster metric that returns the best cluster based on the 
 * number of Startds that are not running jobs.
 * ----------------------------------------------------------------------------
 */
/*@(#) filename, component, release, file version, date changed */
static char sccsid[] = "@(#) src/ll/samples/llcluster/cluster_metric_free_nodes.c, ll.api, ll_rven, rven0635b 1.3 7/25/05";

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <values.h>
#include "llapi.h"

main(int argc, char *argv[])
{
	LL_element *machine=NULL;
	LL_element *queryObject=NULL, *mcluster=NULL;
	LL_element *errObj=NULL;
	LL_cluster_param cluster;
	char *job_id = NULL;
	char *chosen_cluster = NULL;
	char *cluster_name = NULL;
	char **cluster_list = NULL;
	int number_of_cluster_params = 0;
	int search_any = 0; 	
	int min_free_nodes = 0;
	int num_of_free_nodes = 0;
	int valid_query = 0;
	int rc = 0, num = 0, err = 0, value = 0;
	int i = 0, j = 0;

	/* Process cluster metric arguments from LoadLeveler */

	/* Job id to be processed */
	job_id = strdup(argv[1]);

	/* Number of clusters passed in as parameters */
	number_of_cluster_params = atoi(argv[2]);

	/* Get the cluster list to use for metric selection */
	cluster_list=(char **)calloc(number_of_cluster_params,sizeof(char *));	
	j = 0;
	for (i=3; i <= number_of_cluster_params+2; i++) {
		cluster_list[j] = strdup(argv[i]);
		if(!strcmp(cluster_list[j],"any")) {
			search_any = 1;
		}
		j++;
	}

	/* if the reserved word "any" was indicated, go and obtain the
	   remote clusters that are configured for the local cluster */
	if ( search_any ) {
		/* Free existing cluster list */
		for(i=0; i < number_of_cluster_params; i++) {
			free(cluster_list[i]);
		}
		free(cluster_list);

		/* Initialize the query for MClusters */
        	queryObject = ll_query(MCLUSTERS);

        	rc = ll_set_request(queryObject,QUERY_ALL,NULL,ALL_DATA);

        	/* Request the objects from the Schedd daemon */
        	mcluster = ll_get_objs(queryObject,LL_SCHEDD,NULL,&num,&err);

		/* Malloc cluster_list according to the number of mclusters */
		cluster_list=(char **)calloc(num,sizeof(char *));	
		i = 0;

        	/* Did we get a list of objects ? */
        	if (mcluster == NULL) {
			fprintf(stderr,"cluster_metric_free_nodes: Unable to obtain cluster objects for metric request of \"any\".\n");
			free(job_id);
			free(cluster_list);
			exit(-1);
        	} else {
                	/* Loop through the list and process */
			while(mcluster) {
                        	if (ll_get_data(mcluster, LL_MClusterName, &cluster_name)==0) {
					/* Add cluster name to list of
					   clusters to query about
					   number of jobs running */
                                	if (cluster_name) {
                                        	cluster_list[i++] = strdup(cluster_name);
                                        	free(cluster_name);
                                        	cluster_name=NULL;
                                	}
                        	}
     				mcluster = ll_next_obj(queryObject);
                	}
        	}

        	/* free objects obtained */
        	rc = ll_free_objs(queryObject);

        	/* free query element */
        	rc = ll_deallocate(queryObject);

		number_of_cluster_params = i;
	}
	
	/* Initialize LL_cluster_param in order to perform remote query.
	   Query each cluster to get the number of free nodes */
	cluster.cluster_list = (char **)calloc(2,sizeof(char *));

	if(cluster_list[0]) {
		/* default chosen_cluster to first in the list */
		chosen_cluster = cluster_list[0];
	}

	for (i=0; i<number_of_cluster_params; i++) {
		cluster.cluster_list[0] = cluster_list[i];
        	cluster.cluster_list[1] = NULL;
        	cluster.action = CLUSTER_SET;

		rc = ll_cluster(LL_API_VERSION,&errObj,&cluster);

		queryObject = ll_query(MACHINES);

   		rc = ll_set_request(queryObject,QUERY_ALL,NULL,ALL_DATA);

		machine = ll_get_objs(queryObject, LL_CM, NULL, &num, &err);
		if ( err == 0 ) {
			valid_query = 1;
		}

		/* Get data for each machine returned from remote query */
   		while(machine) {
      			rc = ll_get_data(machine, LL_MachineStartdRunningJobs, &value);
				/* Count machines that report 0 jobs running */
         			if (!rc && value == 0) {
						num_of_free_nodes++;
				}
      			machine = ll_next_obj(queryObject);
   		}
   		ll_free_objs(queryObject);
   		ll_deallocate(queryObject);

		cluster.action = CLUSTER_UNSET;
		rc = ll_cluster(LL_API_VERSION,&errObj,&cluster);

		/* If we received valid data from the query,
		   see if this cluster has the greatest
		   number of nodes that are not running jobs */
		if (valid_query) {
			if (num_of_free_nodes > min_free_nodes) {
				min_free_nodes = num_of_free_nodes;
				chosen_cluster = cluster_list[i];
			}
			num_of_free_nodes = 0;
		}
		valid_query = 0;
	}
	fprintf(stdout,"%s", chosen_cluster);

	/* Free existing cluster list */
	for(i=0; i < number_of_cluster_params; i++) {
		free(cluster_list[i]);
	}
	free(cluster_list);
	free(job_id);
	exit(0);
}

==> LoadL-3.4.1.2-0.SLES10-X86.3.4.X.tar.gz: opt/ibmll/LoadL/full/samples/llcluster/cluster_metric_free_nodes.c <==
/*
 * ----------------------------------------------------------------------------
 * This sample program shows how LoadLeveler's Data Access API can be used
 * to create a cluster metric that returns the best cluster based on the 
 * number of Startds that are not running jobs.
 * ----------------------------------------------------------------------------
 */
/*@(#) filename, component, release, file version, date changed */
static char sccsid[] = "@(#) src/ll/samples/llcluster/cluster_metric_free_nodes.c, ll.api, ll_rven2, rven20715a 1.3 7/25/05";

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <values.h>
#include "llapi.h"

main(int argc, char *argv[])
{
	LL_element *machine=NULL;
	LL_element *queryObject=NULL, *mcluster=NULL;
	LL_element *errObj=NULL;
	LL_cluster_param cluster;
	char *job_id = NULL;
	char *chosen_cluster = NULL;
	char *cluster_name = NULL;
	char **cluster_list = NULL;
	int number_of_cluster_params = 0;
	int search_any = 0; 	
	int min_free_nodes = 0;
	int num_of_free_nodes = 0;
	int valid_query = 0;
	int rc = 0, num = 0, err = 0, value = 0;
	int i = 0, j = 0;

	/* Process cluster metric arguments from LoadLeveler */

	/* Job id to be processed */
	job_id = strdup(argv[1]);

	/* Number of clusters passed in as parameters */
	number_of_cluster_params = atoi(argv[2]);

	/* Get the cluster list to use for metric selection */
	cluster_list=(char **)calloc(number_of_cluster_params,sizeof(char *));	
	j = 0;
	for (i=3; i <= number_of_cluster_params+2; i++) {
		cluster_list[j] = strdup(argv[i]);
		if(!strcmp(cluster_list[j],"any")) {
			search_any = 1;
		}
		j++;
	}

	/* if the reserved word "any" was indicated, go and obtain the
	   remote clusters that are configured for the local cluster */
	if ( search_any ) {
		/* Free existing cluster list */
		for(i=0; i < number_of_cluster_params; i++) {
			free(cluster_list[i]);
		}
		free(cluster_list);

		/* Initialize the query for MClusters */
        	queryObject = ll_query(MCLUSTERS);

        	rc = ll_set_request(queryObject,QUERY_ALL,NULL,ALL_DATA);

        	/* Request the objects from the Schedd daemon */
        	mcluster = ll_get_objs(queryObject,LL_SCHEDD,NULL,&num,&err);

		/* Malloc cluster_list according to the number of mclusters */
		cluster_list=(char **)calloc(num,sizeof(char *));	
		i = 0;

        	/* Did we get a list of objects ? */
        	if (mcluster == NULL) {
			fprintf(stderr,"cluster_metric_free_nodes: Unable to obtain cluster objects for metric request of \"any\".\n");
			free(job_id);
			free(cluster_list);
			exit(-1);
        	} else {
                	/* Loop through the list and process */
			while(mcluster) {
                        	if (ll_get_data(mcluster, LL_MClusterName, &cluster_name)==0) {
					/* Add cluster name to list of
					   clusters to query about
					   number of jobs running */
                                	if (cluster_name) {
                                        	cluster_list[i++] = strdup(cluster_name);
                                        	free(cluster_name);
                                        	cluster_name=NULL;
                                	}
                        	}
     				mcluster = ll_next_obj(queryObject);
                	}
        	}

        	/* free objects obtained */
        	rc = ll_free_objs(queryObject);

        	/* free query element */
        	rc = ll_deallocate(queryObject);

		number_of_cluster_params = i;
	}
	
	/* Initialize LL_cluster_param in order to perform remote query.
	   Query each cluster to get the number of free nodes */
	cluster.cluster_list = (char **)calloc(2,sizeof(char *));

	if(cluster_list[0]) {
		/* default chosen_cluster to first in the list */
		chosen_cluster = cluster_list[0];
	}

	for (i=0; i<number_of_cluster_params; i++) {
		cluster.cluster_list[0] = cluster_list[i];
        	cluster.cluster_list[1] = NULL;
        	cluster.action = CLUSTER_SET;

		rc = ll_cluster(LL_API_VERSION,&errObj,&cluster);

		queryObject = ll_query(MACHINES);

   		rc = ll_set_request(queryObject,QUERY_ALL,NULL,ALL_DATA);

		machine = ll_get_objs(queryObject, LL_CM, NULL, &num, &err);
		if ( err == 0 ) {
			valid_query = 1;
		}

		/* Get data for each machine returned from remote query */
   		while(machine) {
      			rc = ll_get_data(machine, LL_MachineStartdRunningJobs, &value);
				/* Count machines that report 0 jobs running */
         			if (!rc && value == 0) {
						num_of_free_nodes++;
				}
      			machine = ll_next_obj(queryObject);
   		}
   		ll_free_objs(queryObject);
   		ll_deallocate(queryObject);

		cluster.action = CLUSTER_UNSET;
		rc = ll_cluster(LL_API_VERSION,&errObj,&cluster);

		/* If we received valid data from the query,
		   see if this cluster has the greatest
		   number of nodes that are not running jobs */
		if (valid_query) {
			if (num_of_free_nodes > min_free_nodes) {
				min_free_nodes = num_of_free_nodes;
				chosen_cluster = cluster_list[i];
			}
			num_of_free_nodes = 0;
		}
		valid_query = 0;
	}
	fprintf(stdout,"%s", chosen_cluster);

	/* Free existing cluster list */
	for(i=0; i < number_of_cluster_params; i++) {
		free(cluster_list[i]);
	}
	free(cluster_list);
	free(job_id);
	exit(0);
}
